from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.parent import Parent
from sevdesk.converters.category import Category

class Contact(BaseModel):
    """Contact model"""

    name: Optional[str] = Field(default=None, description="The organization name.<br> Be aware that the type of contact will depend on this attribute.<br> If it holds a value, the contact will be regarded as an organization.")
    status: Optional[int] = Field(default=None, description="Defines the status of the contact. 100 <-> Lead - 500 <-> Pending - 1000 <-> Active.")
    customerNumber: Optional[str] = Field(default=None, description="The customer number")
    parent: Optional[Parent] = Field(default=None, description="The parent contact to which this contact belongs. Must be an organization.")
    surename: Optional[str] = Field(default=None, description="The <b>first</b> name of the contact.<br> Yeah... not quite right in literally every way. We know.<br> Not to be used for organizations.")
    familyname: Optional[str] = Field(default=None, description="The last name of the contact.<br> Not to be used for organizations.")
    titel: Optional[str] = Field(default=None, description="A non-academic title for the contact. Not to be used for organizations.")
    category: Category = Field(description="Category of the contact.<br> For more information, see <a href='https://my.sevdesk.de/apiOverview/index.html#/doc-contacts#types'>here</a>.")
    description: Optional[str] = Field(default=None, description="A description for the contact.")
    academicTitle: Optional[str] = Field(default=None, description="A academic title for the contact. Not to be used for organizations.")
    gender: Optional[str] = Field(default=None, description="Gender of the contact.<br> Not to be used for organizations.")
    name2: Optional[str] = Field(default=None, description="Second name of the contact.<br> Not to be used for organizations.")
    birthday: Optional[str] = Field(default=None, description="Birthday of the contact.<br> Not to be used for organizations.")
    vatNumber: Optional[str] = Field(default=None, description="Vat number of the contact.")
    bankAccount: Optional[str] = Field(default=None, description="Bank account number (IBAN) of the contact.")
    bankNumber: Optional[str] = Field(default=None, description="Bank number of the bank used by the contact.")
    defaultCashbackTime: Optional[int] = Field(default=None, description="Absolute time in days which the contact has to pay his invoices and subsequently get a cashback.")
    defaultCashbackPercent: Optional[float] = Field(default=None, description="Percentage of the invoice sum the contact gets back if he paid invoices in time.")
    defaultTimeToPay: Optional[int] = Field(default=None, description="The payment goal in days which is set for every invoice of the contact.")
    taxNumber: Optional[str] = Field(default=None, description="The tax number of the contact.")
    taxOffice: Optional[str] = Field(default=None, description="The tax office of the contact (only for greek customers).")
    exemptVat: Optional[bool] = Field(default=None, description="Defines if the contact is freed from paying vat.")
    defaultDiscountAmount: Optional[float] = Field(default=None, description="The default discount the contact gets for every invoice.<br> Depending on defaultDiscountPercentage attribute, in percent or absolute value.")
    defaultDiscountPercentage: Optional[bool] = Field(default=None, description="Defines if the discount is a percentage (true) or an absolute value (false).")
    buyerReference: Optional[str] = Field(default=None, description="Buyer reference of the contact.")
    governmentAgency: Optional[bool] = Field(default=None, description="Defines whether the contact is a government agency (true) or not (false).")
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.communicationway import CommunicationWay
from sevdesk.models.communicationwayresponse import CommunicationWayResponse
from sevdesk.models.communicationwayupdate import CommunicationWayUpdate

class CommunicationWayController(BaseController):

    @BaseController.get("/CommunicationWay")
    def getCommunicationWays(self, contact_id: Optional[str] = None, contact_objectName: Optional[str] = None, main: Optional[str] = None, type_: Optional[str] = None) -> list[CommunicationWayResponse]:
        """Retrieve communication ways"""
        return (yield)

    @BaseController.post("/CommunicationWay")
    def createCommunicationWay(self, body: CommunicationWay) -> CommunicationWayResponse:
        """Create a new contact communication way"""
        return (yield)

    @BaseController.get("/CommunicationWay/{communicationWayId}")
    def getCommunicationWayById(self, communicationWayId: int) -> list[CommunicationWayResponse]:
        """Find communication way by ID"""
        return (yield)

    @BaseController.delete("/CommunicationWay/{communicationWayId}")
    def deleteCommunicationWay(self, communicationWayId: int):
        """Deletes a communication way"""
        return (yield)

    @BaseController.put("/CommunicationWay/{communicationWayId}")
    def UpdateCommunicationWay(self, communicationWayId: int, body: CommunicationWayUpdate) -> CommunicationWayResponse:
        """Update a existing communication way"""
        return (yield)

    @BaseController.get("/CommunicationWayKey")
    def getCommunicationWayKeys(self):
        """Retrieve communication way keys"""
        return (yield)




from typing import Optional, Any
from pydantic import BaseModel, Field

from sevdesk.converters.discountdelete import DiscountDelete

class SaveInvoice(BaseModel):
    invoice: str
    invoicePosSave: Optional[Any] = None
    invoicePosDelete: Optional[str] = None
    filename: Optional[str] = Field(default=None, description="Filename of a previously upload file which should be attached.")
    discountSave: Optional[Any] = None
    discountDelete: Optional[DiscountDelete] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.sevclient import SevClient

class AccountingContactResponse(BaseModel):
    """Accounting contact model"""

    id_: Optional[str] = Field(default=None, alias="id", description="The accounting contact id")
    objectName: Optional[str] = Field(default=None, description="The accounting contact object name")
    create: Optional[str] = Field(default=None, description="Date of accounting contact creation")
    update: Optional[str] = Field(default=None, description="Date of last accounting contact update")
    contact: Optional[Contact] = Field(default=None, description="The contact to which this accounting contact belongs.")
    sevClient: Optional[SevClient] = Field(default=None, description="Client to which accounting contact belongs. Will be filled automatically")
    debitorNumber: Optional[str] = Field(default=None, description="Debitor number of the accounting contact.")
    creditorNumber: Optional[str] = Field(default=None, description="Creditor number of the accounting contact.")
    class Config:
        populate_by_name = True


"""
CreditNoteHelper - High-Level Gutschriften-Verwaltung

Gutschriften (CreditNotes) werden verwendet fuer:
- Stornorechnungen
- Teilstornos
- Gutschriften an Kunden

Beispiele:
    # Alle Gutschriften auflisten
    creditnotes = sevdesk.creditNoteHelper.list()

    # Gutschrift per ID abrufen
    cn = sevdesk.creditNoteHelper.find_by_id(12345)

    # PDF herunterladen
    pdf = sevdesk.creditNoteHelper.get_pdf(12345)
"""

from datetime import datetime, timedelta
from typing import Optional, List
from sevdesk.models.creditnoteresponse import CreditNoteResponse


# CreditNote Status
CREDITNOTE_STATUS = {
    '100': 'Draft (Entwurf)',
    '200': 'Open (Offen)',
    '1000': 'Paid (Bezahlt)',
}


class CreditNoteHelper:
    """Helper-Klasse fuer Gutschriften-Operationen"""

    def __init__(self, client):
        self.client = client

    def list(self, contact_id: int = None, status: str = None,
             start_date: str = None, end_date: str = None,
             credit_note_number: str = None) -> List[CreditNoteResponse]:
        """
        Listet Gutschriften auf.

        Args:
            contact_id: Filter nach Kontakt
            status: Filter nach Status ('draft', 'open', 'paid' oder Code)
            start_date: Startdatum (YYYY-MM-DD)
            end_date: Enddatum (YYYY-MM-DD)
            credit_note_number: Filter nach Gutschriftennummer

        Returns:
            Liste von CreditNoteResponse
        """
        # Status-Mapping
        status_map = {
            'draft': '100',
            'open': '200',
            'paid': '1000',
        }
        status_value = None
        if status:
            status_value = status_map.get(status.lower(), status)

        # Datum in Timestamp
        start_ts = self._date_to_timestamp(start_date) if start_date else None
        end_ts = self._date_to_timestamp(end_date) if end_date else None

        try:
            creditnotes = self.client.creditnote.getCreditNotes(
                contact_id=contact_id,
                contact_objectName="Contact" if contact_id else None,
                status=status_value,
                startDate=start_ts,
                endDate=end_ts,
                creditNoteNumber=credit_note_number
            )
            return creditnotes if creditnotes else []
        except Exception:
            return []

    def find_by_id(self, creditnote_id: int) -> Optional[CreditNoteResponse]:
        """Ruft eine Gutschrift per ID ab"""
        try:
            result = self.client.creditnote.getcreditNoteById(creditnote_id)
            if isinstance(result, list) and len(result) > 0:
                return result[0]
            return result
        except Exception:
            return None

    def find_by_number(self, creditnote_number: str) -> Optional[CreditNoteResponse]:
        """Sucht nach Gutschriftennummer"""
        creditnotes = self.list(credit_note_number=creditnote_number)
        return creditnotes[0] if creditnotes else None

    def get_drafts(self) -> List[CreditNoteResponse]:
        """Ruft alle Entwuerfe ab"""
        return self.list(status='draft')

    def get_open(self) -> List[CreditNoteResponse]:
        """Ruft alle offenen Gutschriften ab"""
        return self.list(status='open')

    def get_paid(self) -> List[CreditNoteResponse]:
        """Ruft alle bezahlten Gutschriften ab"""
        return self.list(status='paid')

    def get_recent(self, days: int = 30) -> List[CreditNoteResponse]:
        """Ruft Gutschriften der letzten X Tage ab"""
        end_date = datetime.now().strftime("%Y-%m-%d")
        start_date = (datetime.now() - timedelta(days=days)).strftime("%Y-%m-%d")
        return self.list(start_date=start_date, end_date=end_date)

    def get_by_contact(self, contact_id: int) -> List[CreditNoteResponse]:
        """Ruft alle Gutschriften eines Kontakts ab"""
        return self.list(contact_id=contact_id)

    def get_pdf(self, creditnote_id: int, download: bool = True) -> Optional[bytes]:
        """Laedt PDF einer Gutschrift herunter"""
        try:
            return self.client.creditnote.creditNoteGetPdf(creditnote_id, download=download)
        except Exception:
            return None

    def mark_as_sent(self, creditnote_id: int) -> Optional[CreditNoteResponse]:
        """Markiert Gutschrift als versendet"""
        try:
            return self.client.creditnote.creditNoteSendBy(creditnote_id)
        except Exception:
            return None

    def reset_to_draft(self, creditnote_id: int) -> bool:
        """Setzt Status zurueck auf Entwurf"""
        try:
            self.client.creditnote.creditNoteResetToDraft(creditnote_id)
            return True
        except Exception:
            return False

    def reset_to_open(self, creditnote_id: int) -> bool:
        """Setzt Status zurueck auf Offen"""
        try:
            self.client.creditnote.creditNoteResetToOpen(creditnote_id)
            return True
        except Exception:
            return False

    def enshrine(self, creditnote_id: int) -> bool:
        """Schreibt Gutschrift fest (nicht mehr aenderbar)"""
        try:
            self.client.creditnote.creditNoteEnshrine(creditnote_id)
            return True
        except Exception:
            return False

    def delete(self, creditnote_id: int) -> bool:
        """Loescht eine Gutschrift (nur Entwuerfe)"""
        try:
            self.client.creditnote.deletecreditNote(creditnote_id)
            return True
        except Exception:
            return False

    def get_status_label(self, status: str) -> str:
        """Gibt Status-Label zurueck"""
        return CREDITNOTE_STATUS.get(str(status), f'Unknown ({status})')

    def calculate_totals(self, creditnotes: List[CreditNoteResponse]) -> dict:
        """Berechnet Summen"""
        net = 0.0
        tax = 0.0
        gross = 0.0
        for cn in creditnotes:
            if cn.sumNet:
                net += float(cn.sumNet)
            if cn.sumTax:
                tax += float(cn.sumTax)
            if cn.sumGross:
                gross += float(cn.sumGross)
        return {'net': net, 'tax': tax, 'gross': gross, 'count': len(creditnotes)}

    def _date_to_timestamp(self, date_str: str) -> Optional[int]:
        """Konvertiert Datum in Timestamp"""
        try:
            if date_str.isdigit():
                return int(date_str)
            dt = datetime.strptime(date_str, "%Y-%m-%d")
            return int(dt.timestamp())
        except (ValueError, AttributeError):
            return None



from typing import Optional
from pydantic import BaseModel, Field


class CreateClearingAccount(BaseModel):
    name: Optional[str] = Field(default=None, description="Name of the check account")
    accountingNumber: Optional[int] = Field(default=None, description="The booking account used for this clearing account, e.g. 3320 in SKR04 and 1723 in SKR03. Must be unique among all your CheckAccounts. Ask your tax consultant what to choose.")
    class Config:
        populate_by_name = True


"""
Undocumented Invoice Controller

Erweitert die Standard-Invoice-Funktionen mit nicht-offiziell dokumentierten Features.
"""

from sevdesk.base.basecontroller import BaseController
from sevdesk.controllers.invoice_controller import InvoiceController as BaseInvoiceController
from sevdesk.models.invoice import Invoice
from sevdesk.models.invoiceresponse import InvoiceResponse


class InvoiceController(BaseInvoiceController):
    """
    Erweiterte Invoice-Controller mit zusaetzlichen Funktionen.

    Zusaetzliche Methoden:
    - createInvoice() - Direktes Erstellen von Rechnungen (nicht nur Factory)
    """

    @BaseController.post("/Invoice")
    def createInvoice(self, body: Invoice) -> InvoiceResponse:
        """
        Erstellt eine Rechnung direkt via POST /Invoice.

        Dies ist nicht offiziell dokumentiert, aber funktioniert und ist
        einfacher als die Factory-Methode.

        Args:
            body: Invoice-Model mit allen Rechnungsdaten

        Returns:
            InvoiceResponse mit der gespeicherten Rechnung
        """
        return (yield)



"""
LetterHelper - High-Level Brief-Verwaltung

Beispiele:
    letter = sevdesk.letterHelper.new(
        contact=contact,
        header='Betreff des Briefes',
        text='<p>Inhalt des Briefes</p>'
    )
    letter.save()
    letter.render()
    pdf = letter.getPDF()
"""

import time
from datetime import datetime
from typing import Optional
from sevdesk.undocumented.models.letter import Letter
from sevdesk.converters.contact import Contact
from sevdesk.converters.contactperson import ContactPerson


class LetterExt:
    """
    Erweiterte Brief-Klasse mit High-Level Funktionen.

    Beispiel:
        letter = LetterExt(...)
        letter.save()
        letter.render()
        pdf = letter.getPDF()
    """

    def __init__(self, client, contact, header: str, text: str,
                 address: str = None, contactPerson_id: int = None,
                 contactPerson_name: str = "SevUser", status: str = "100",
                 letterDate: str = None):
        self._client = client
        self._saved_id = None

        # Contact normalisieren
        if isinstance(contact, int):
            self._contact = Contact(id_=contact, objectName="Contact")
        elif hasattr(contact, 'id_'):
            self._contact = Contact(id_=contact.id_, objectName="Contact")
        else:
            self._contact = contact

        self._header = header
        self._text = text
        self._address = address
        self._status = status

        # Datum: Unix-Timestamp oder heute
        if letterDate:
            self._letterDate = letterDate
        else:
            self._letterDate = str(int(datetime.now().timestamp()))

        # ContactPerson (SevUser)
        if contactPerson_id:
            self._contactPerson = ContactPerson(id_=contactPerson_id, objectName=contactPerson_name)
        else:
            self._contactPerson = None

    def _get_default_contact_person(self) -> ContactPerson:
        """Holt den ersten verfuegbaren SevUser als ContactPerson"""
        try:
            users = self._client.undocumented.sevuser.getSevUsers(limit=1)
            if users and len(users) > 0:
                user = users[0]
                user_id = user.get('id') or user.get('id_')
                if user_id:
                    return ContactPerson(id_=int(user_id), objectName="SevUser")
        except Exception:
            pass
        raise RuntimeError(
            "Kein SevUser gefunden. Bitte contactPerson_id explizit angeben."
        )

    def save(self) -> 'LetterExt':
        """
        Speichert den Brief auf sevDesk.

        Returns:
            self mit gesetzter ID (fuer Method-Chaining)
        """
        if not self._client:
            raise RuntimeError("Client nicht gesetzt")

        # ContactPerson ist required - fallback auf ersten User
        if not self._contactPerson:
            self._contactPerson = self._get_default_contact_person()

        letter = Letter(
            contact=self._contact,
            letterDate=self._letterDate,
            header=self._header,
            status=self._status,
            contactPerson=self._contactPerson,
            text=self._text,
            address=self._address,
        )

        result = self._client.undocumented.letter.createLetter(body=letter)

        if hasattr(result, 'id_'):
            self._saved_id = int(result.id_)
        elif isinstance(result, dict):
            self._saved_id = int(result.get('id') or result.get('id_'))

        if not self._saved_id:
            raise RuntimeError("Konnte Letter-ID nicht aus Response extrahieren")

        return self

    def render(self, forceReload: bool = False) -> 'LetterExt':
        """
        Rendert den Brief als PDF.

        Returns:
            self (fuer Method-Chaining)
        """
        if not self._saved_id:
            raise RuntimeError("Brief muss zuerst gespeichert werden (save())")

        self._client.undocumented.letter.letterRender(
            letterId=self._saved_id,
            forceReload=forceReload,
            getAsPdf=True
        )
        return self

    def getPDF(self, download: bool = True) -> Optional[bytes]:
        """
        Laedt die PDF des Briefes herunter.

        Args:
            download: Ob die Datei heruntergeladen werden soll

        Returns:
            PDF-Inhalt (bytes)
        """
        if not self._saved_id:
            raise RuntimeError("Brief muss zuerst gespeichert werden (save())")

        pdf_content = self._client.undocumented.letter.letterGetPdf(
            letterId=self._saved_id,
            download=download
        )
        return pdf_content

    def setParameter(self, key: str, value: str) -> 'LetterExt':
        """
        Aendert einen Design-Parameter des Briefes.

        Args:
            key: Parameter-Name (z.B. 'logoSize', 'color')
            value: Neuer Wert

        Returns:
            self (fuer Method-Chaining)
        """
        if not self._saved_id:
            raise RuntimeError("Brief muss zuerst gespeichert werden (save())")

        self._client.undocumented.letter.changeLetterParameter(
            letterId=self._saved_id,
            key=key,
            value=value,
            getAsPdf=True
        )
        return self

    def markAsSent(self, sendType: str = "VPDF") -> 'LetterExt':
        """
        Markiert den Brief als versendet.

        Args:
            sendType: Versandart (default: VPDF)

        Returns:
            self (fuer Method-Chaining)
        """
        if not self._saved_id:
            raise RuntimeError("Brief muss zuerst gespeichert werden (save())")

        self._client.undocumented.letter.letterSendBy(
            letterId=self._saved_id,
            sendType=sendType
        )
        return self

    @property
    def id(self) -> Optional[int]:
        """Die ID des gespeicherten Briefes"""
        return self._saved_id


class LetterHelper:
    """Helper-Klasse fuer Brief-Operationen auf hohem Level"""

    def __init__(self, client):
        self.client = client

    def new(self, contact, header: str, text: str,
            address: str = None, contactPerson_id: int = None,
            contactPerson_name: str = "SevUser", status: str = "100",
            letterDate: str = None) -> LetterExt:
        """
        Erstellt einen neuen Brief (noch nicht gespeichert).

        Args:
            contact: Contact-Objekt, ContactResponse oder Contact-ID
            header: Betreff des Briefes
            text: HTML-Inhalt des Briefes
            address: Empfaengeradresse (mehrzeilig)
            contactPerson_id: SevUser-ID (optional, wird automatisch ermittelt)
            contactPerson_name: Object-Name (default: SevUser)
            status: Status ('100'=Draft)
            letterDate: Datum als Unix-Timestamp (default: heute)

        Returns:
            LetterExt-Objekt (noch nicht gespeichert)
        """
        return LetterExt(
            client=self.client,
            contact=contact,
            header=header,
            text=text,
            address=address,
            contactPerson_id=contactPerson_id,
            contactPerson_name=contactPerson_name,
            status=status,
            letterDate=letterDate,
        )

    def find_by_id(self, letter_id: int):
        """
        Ruft einen Brief per ID ab.

        Args:
            letter_id: ID des Briefes

        Returns:
            LetterResponse oder None
        """
        try:
            result = self.client.undocumented.letter.getLetterById(letterId=letter_id)
            if isinstance(result, list) and len(result) > 0:
                return result[0]
            return result
        except Exception:
            return None

    def list(self, contact_id: int = None, limit: int = 100):
        """
        Listet Briefe auf.

        Args:
            contact_id: Optional: nur fuer einen Kontakt
            limit: Max. Anzahl

        Returns:
            Liste von LetterResponse-Objekten
        """
        try:
            letters = self.client.undocumented.letter.getLetters(
                contact_id=contact_id,
                contact_objectName="Contact" if contact_id else None,
                limit=limit
            )
            return letters if letters else []
        except Exception:
            return []



from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient

class ContactCustomFieldSettingResponse(BaseModel):
    """contact fields model"""

    id_: Optional[str] = Field(default=None, alias="id", description="Id of the contact field")
    objectName: Optional[str] = Field(default=None, description="Internal object name which is 'ContactCustomFieldSetting'.")
    create: Optional[str] = Field(default=None, description="Date of contact field creation")
    update: Optional[str] = Field(default=None, description="Date of contact field updated")
    sevClient: Optional[SevClient] = Field(default=None, description="Client to which invoice belongs. Will be filled automatically")
    name: Optional[str] = Field(default=None, description="name of the contact fields")
    identifier: Optional[str] = Field(default=None, description="Unique identifier for the contact field")
    description: Optional[str] = Field(default=None, description="The description of the contact field")
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient

class TagResponse(BaseModel):
    """tag model"""

    id_: Optional[str] = Field(default=None, alias="id", description="Id of the tag")
    objectName: Optional[str] = Field(default=None, description="Internal object name which is 'Tag'.")
    additionalInformation: Optional[str] = None
    create: Optional[str] = Field(default=None, description="Date of tag creation")
    name: Optional[str] = Field(default=None, description="name of the tag")
    sevClient: Optional[SevClient] = Field(default=None, description="Client to which invoice belongs. Will be filled automatically")
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field


class CreateFileImportAccount(BaseModel):
    name: Optional[str] = Field(default=None, description="Name of the check account")
    importType: Optional[str] = Field(default=None, description="Import type. Transactions can be imported by this method on the check account.")
    accountingNumber: Optional[int] = Field(default=None, description="The booking account used for this bank account, e.g. 1800 in SKR04 and 1200 in SKR03. Must be unique among all your CheckAccounts. Ignore to use a sensible default.")
    iban: Optional[str] = Field(default=None, description="IBAN of the bank account, without spaces")
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact

class AccountingContact(BaseModel):
    """Accounting contact model"""

    contact: Contact = Field(description="The contact to which this accounting contact belongs.")
    debitorNumber: Optional[int] = Field(default=None, description="Debitor number of the accounting contact.")
    creditorNumber: Optional[int] = Field(default=None, description="Creditor number of the accounting contact.")
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

class Error(BaseModel):
    message: Optional[str] = None
    exceptionUUID: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.checkaccount import CheckAccount
from sevdesk.converters.sourcetransaction import SourceTransaction
from sevdesk.converters.targettransaction import TargetTransaction

class CheckAccountTransaction(BaseModel):
    """CheckAccountTransaction model. Responsible for the transactions on payment accounts."""

    id_: Optional[int] = Field(default=None, alias="id", description="The check account transaction id")
    objectName: Optional[str] = Field(default=None, description="The check account transaction object name")
    create: Optional[str] = Field(default=None, description="Date of check account transaction creation")
    update: Optional[str] = Field(default=None, description="Date of last check account transaction update")
    sevClient: Optional[SevClient] = Field(default=None, description="Client to which check account transaction belongs. Will be filled automatically")
    valueDate: str = Field(description="Date the check account transaction was booked")
    entryDate: Optional[str] = Field(default=None, description="Date the check account transaction was imported")
    paymtPurpose: Optional[str] = Field(default=None, description="The purpose of the transaction")
    amount: float = Field(description="Amount of the transaction")
    payeePayerName: Optional[str] = Field(description="Name of the other party")
    payeePayerAcctNo: Optional[str] = Field(default=None, description="IBAN or account number of the other party")
    payeePayerBankCode: Optional[str] = Field(default=None, description="BIC or bank code of the other party")
    checkAccount: CheckAccount = Field(description="The check account to which the transaction belongs")
    status: int = Field(description="Status of the check account transaction.<br> 100 <-> Created<br> 200 <-> Linked<br> 300 <-> Private<br> 400 <-> Booked")
    sourceTransaction: Optional[SourceTransaction] = Field(default=None, description="The check account transaction serving as the source of the rebooking")
    targetTransaction: Optional[TargetTransaction] = Field(default=None, description="The check account transaction serving as the target of the rebooking")
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient

class CreateFileImportAccountResponse(BaseModel):
    """CheckAccount model. Showing the properties relevant to file import accounts."""

    id_: Optional[str] = Field(default=None, alias="id", description="The check account id")
    objectName: Optional[str] = Field(default=None, description="The check account object name, always 'CheckAccount'")
    create: Optional[str] = Field(default=None, description="Date of check account creation")
    update: Optional[str] = Field(default=None, description="Date of last check account update")
    sevClient: Optional[SevClient] = Field(default=None, description="Client to which check account belongs. Will be filled automatically")
    name: Optional[str] = Field(default=None, description="Name of the check account")
    iban: Optional[str] = Field(default=None, description="The IBAN of the account")
    type_: Optional[str] = Field(default=None, alias="type", description="The type of the check account. Account with a CSV or MT940 import are regarded as online.")
    importType: Optional[str] = Field(default=None, description="Import type, for accounts that are type 'online' but not connected to a data provider. Transactions can be imported by this method on the check account.")
    currency: Optional[str] = Field(default=None, description="The currency of the check account.")
    defaultAccount: Optional[str] = Field(default=None, description="Defines if this check account is the default account.")
    status: Optional[str] = Field(default=None, description="Status of the check account. 0 <-> Archived - 100 <-> Active")
    autoMapTransactions: Optional[str] = Field(default=None, description="Defines if transactions on this account are automatically mapped to invoice and vouchers when imported if possible.")
    accountingNumber: Optional[str] = Field(default=None, description="The booking account used for this bank account, e.g. 1800 in SKR04 and 1200 in SKR03. Must be unique among all your CheckAccounts. Ignore to use a sensible default.")
    class Config:
        populate_by_name = True


"""
Undocumented Order Controller

Erweitert die Standard-Order-Funktionen mit nicht-offiziell dokumentierten Features.
"""

from sevdesk.base.basecontroller import BaseController
from sevdesk.controllers.order_controller import OrderController as BaseOrderController
from sevdesk.models.order import Order
from sevdesk.models.orderresponse import OrderResponse


class OrderController(BaseOrderController):
    """
    Erweiterte Order-Controller mit zusaetzlichen Funktionen.

    Zusaetzliche Methoden:
    - createOrderDirect() - Direktes Erstellen von Angeboten/Auftraegen (nicht Factory)
    """

    @BaseController.post("/Order")
    def createOrderDirect(self, body: Order) -> OrderResponse:
        """
        Erstellt ein Angebot/Auftrag direkt via POST /Order.

        Dies ist nicht offiziell dokumentiert, aber funktioniert und ist
        einfacher als die Factory-Methode (die fehlerhafte SaveOrder-Model hat).

        Args:
            body: Order-Model mit allen Daten

        Returns:
            OrderResponse mit dem gespeicherten Angebot/Auftrag
        """
        return (yield)



from sevdesk.base.basecontroller import BaseController
from typing import Optional, Any


class SevUserController(BaseController):
    """Controller fuer SevUser-Operationen (undocumented API)"""

    @BaseController.get("/SevUser")
    def getSevUsers(self, limit: Optional[int] = None, offset: Optional[int] = None) -> list[dict]:
        """Ruft alle SevUser des Accounts ab"""
        return (yield)

    @BaseController.get("/SevUser/{sevUserId}")
    def getSevUserById(self, sevUserId: int) -> dict:
        """Ruft einen spezifischen SevUser ab"""
        return (yield)



from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.contact import Contact

class ContactCustomFieldResponse(BaseModel):
    """contact fields model"""

    id_: Optional[str] = Field(default=None, alias="id", description="id of the contact field")
    objectName: Optional[str] = Field(default=None, description="Internal object name which is 'ContactCustomField'.")
    create: Optional[str] = Field(default=None, description="Date of contact field creation")
    update: Optional[str] = Field(default=None, description="Date of contact field update")
    sevClient: Optional[SevClient] = Field(default=None, description="Client to which contact field belongs. Will be filled automatically")
    contact: Optional[Contact] = Field(default=None, description="name of the contact")
    contactCustomFieldSetting: Optional[dict] = Field(default=None, description="the contact custom field setting")
    value: Optional[str] = Field(default=None, description="The value of the contact field")
    class Config:
        populate_by_name = True